/*
 * ExactMultiNondeterministicSubsequenceAutomaton.cpp
 *
 *  Created on: 7. 4. 2015
 *      Author: Jan Travnicek
 */

#include "ExactMultiNondeterministicSubsequenceAutomaton.h"
#include <string/LinearString.h>
#include <string/Epsilon.h>
#include "../../common/ContainerConverter.hpp"

namespace stringology {

namespace exact {

automaton::Automaton ExactMultiNondeterministicSubsequenceAutomaton::construct(const std::set<string::String>& texts) {
	return automaton::Automaton(ExactMultiNondeterministicSubsequenceAutomaton::construct(common::ContainerConverter<std::set<string::LinearString < >>, std::set<string::String>, string::LinearString < >>::convert(texts)));
}

automaton::EpsilonNFA < > ExactMultiNondeterministicSubsequenceAutomaton::construct(const std::set<string::LinearString < >>& texts) {
	automaton::EpsilonNFA < > res(label::labelFrom(0));
	res.addFinalState(label::labelFrom(0));

	int j = 0;
	for(const string::LinearString < >& text : texts) {
		res.addInputSymbols(text.getAlphabet());

		res.addState(label::labelFrom(0, j));
		res.addFinalState(label::labelFrom(0, j));
		res.addTransition(label::labelFrom(0), label::labelFrom(0, j));

		int i = 1;
		for(const alphabet::Symbol& symbol : text.getContent()) {
			res.addState(label::labelFrom(i, j));
			res.addFinalState(label::labelFrom(i, j));

			res.addTransition(label::labelFrom(i-1, j), symbol, label::labelFrom(i, j));
			res.addTransition(label::labelFrom(i-1, j), label::labelFrom(i, j));
			i++;
		}
		j++;
	}

	return res;
}

} /* namespace exact */

} /* namespace stringology */
