/*
 * ExactSubsequenceAutomaton.cpp
 *
 *  Created on: 7. 4. 2015
 *      Author: Jan Travnicek
 */

#include "ExactSubsequenceAutomaton.h"
#include <string/LinearString.h>
#include <string/Epsilon.h>

#include <deque>

namespace stringology {

namespace exact {

automaton::Automaton ExactSubsequenceAutomaton::construct(const string::String& text) {
	return dispatch(text.getData());
}

automaton::DFA<> ExactSubsequenceAutomaton::construct(const string::LinearString < >& text) {
	std::map<alphabet::Symbol, int> f;
	for(const alphabet::Symbol& symbol : text.getAlphabet()) {
		f[symbol] = 0;
	}

	automaton::DFA<> res(label::labelFrom(0));
	res.addFinalState(label::labelFrom(0));
	res.setInputAlphabet(text.getAlphabet());

	int i = 1;
	for(const alphabet::Symbol& symbol : text.getContent()) {
		res.addState(label::labelFrom(i));
		res.addFinalState(label::labelFrom(i));

		for(int j = f[symbol]; j < i; j++) {
			res.addTransition(label::labelFrom(j), symbol, label::labelFrom(i));
		}
		f[symbol] = i;
		i++;
	}
	return res;
}

auto ExactSubsequenceAutomatonLinearString = ExactSubsequenceAutomaton::RegistratorWrapper<automaton::DFA<>, string::LinearString < >>(ExactSubsequenceAutomaton::construct);

} /* namespace exact */

} /* namespace stringology */
