#include <list>
#include "minimizeBrzozowskiTest.h"

#include "automaton/simplify/MinimizeBrzozowski.h"
#include "automaton/simplify/Minimize.h"
#include "automaton/simplify/Trim.h"
#include "automaton/simplify/Normalize.h"

#define CPPUNIT_IMPLY(x, y) CPPUNIT_ASSERT(!(x) || (y))

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( minimizeBrzozowskiTest, "automaton" );
CPPUNIT_TEST_SUITE_REGISTRATION( minimizeBrzozowskiTest );

void minimizeBrzozowskiTest::setUp() {
}

void minimizeBrzozowskiTest::tearDown() {
}

void minimizeBrzozowskiTest::testMinimizeBrzozowski() {
  automaton::DFA < > automaton(label::labelFrom(1));

  automaton.addState(label::labelFrom(1));
  automaton.addState(label::labelFrom(2));
  automaton.addState(label::labelFrom(3));
  automaton.addInputSymbol(alphabet::symbolFrom("a"));
  automaton.addInputSymbol(alphabet::symbolFrom("b"));
  
  automaton.addTransition(label::labelFrom(1), alphabet::symbolFrom("a"), label::labelFrom(2));
  automaton.addTransition(label::labelFrom(2), alphabet::symbolFrom("b"), label::labelFrom(1));

  automaton.addFinalState(label::labelFrom(3));
  
  automaton::DFA<> minimizedHopcroft = automaton::simplify::Minimize::minimize(automaton);
  automaton::DFA<> minimizedBrzozowski = automaton::simplify::MinimizeBrzozowski::minimize(automaton);

  CPPUNIT_ASSERT(minimizedHopcroft.getStates().size() == 3);
  CPPUNIT_ASSERT(automaton::simplify::Normalize::normalize(automaton::simplify::Trim::trim(minimizedHopcroft)) == automaton::simplify::Normalize::normalize(automaton::simplify::Trim::trim(minimizedBrzozowski)));
}
